import unittest

from sense_hat_matrix.LedMatrixUtil import list_to_matrix
from sense_hat_matrix.LedMatrixUtil import shift_matrix_to_left
from test.PrettyPrint import pretty_print


o = [0, 0, 0]  # Black
R = [255, 0, 0]  # Red
G = [0, 255, 0]  # Green


class LedMatrixUtilSpec(unittest.TestCase):

    def test_should_make_matrix_from_list(self):
        name_of_test = "should_make_matrix_from_list"
        result = list_to_matrix([
            o, o, o, o, o, o, o, R,
            o, o, o, o, o, o, o, R,
            o, o, o, o, o, o, o, R,
            o, o, o, o, o, o, o, R,
            o, o, o, o, o, o, o, R,
            o, o, o, o, o, o, o, R,
            o, o, o, o, o, o, o, R,
            o, o, o, o, o, o, o, R
            ])
        pretty_print(name_of_test, result)
        assert result == [
            [o, o, o, o, o, o, o, R],
            [o, o, o, o, o, o, o, R],
            [o, o, o, o, o, o, o, R],
            [o, o, o, o, o, o, o, R],
            [o, o, o, o, o, o, o, R],
            [o, o, o, o, o, o, o, R],
            [o, o, o, o, o, o, o, R],
            [o, o, o, o, o, o, o, R]
            ], name_of_test

    def test_should_left_shift_matrix_by_one(self):
        name_of_test = "should_left_shift_matrix_by_one"
        matrix = [
            [o, o, o, o, o, o, o, R],
            [o, o, o, o, o, o, o, R],
            [o, o, o, o, o, o, o, R],
            [o, o, o, o, o, o, o, R],
            [o, o, o, o, o, o, o, R],
            [o, o, o, o, o, o, o, R],
            [o, o, o, o, o, o, o, R],
            [o, o, o, o, o, o, o, R]
            ]
        result = shift_matrix_to_left(matrix)
        pretty_print(name_of_test, result)
        assert result == [
            [o, o, o, o, o, o, R, o],
            [o, o, o, o, o, o, R, o],
            [o, o, o, o, o, o, R, o],
            [o, o, o, o, o, o, R, o],
            [o, o, o, o, o, o, R, o],
            [o, o, o, o, o, o, R, o],
            [o, o, o, o, o, o, R, o],
            [o, o, o, o, o, o, R, o]
            ], name_of_test


if __name__ == "__main__":
    unittest.main()



import unittest

from sense_hat_matrix.GraphUtil import rescale


class GraphUtilSpec(unittest.TestCase):

    def test_should_rescale_less_than_min(self):
        result = rescale(10, 20, 2)
        print(result)
        assert result == 1, "should_rescale_less_than_min"

    def test_should_rescale_more_than_max(self):
        result = rescale(10, 20, 25)
        print(result)
        assert result == 8, "should_rescale_more_than_max"

    def test_should_rescale_min(self):
        result = rescale(10, 20, 10)
        print(result)
        assert result == 1, "should_rescale_min"

    def test_should_rescale_max(self):
        result = rescale(10, 20, 20)
        print(result)
        assert result == 8, "should_rescale_max"

    def test_should_rescale_middle(self):
        result = rescale(10, 20, 15)
        print(result)
        assert result == 4.5, "should_rescale_middle"


if __name__ == "__main__":
    unittest.main()



# from sense_hat import SenseHat
# from Graph import Graph
#
# min_temp = 20
# max_temp = 24
# sense = SenseHat()
# g = Graph(min_temp, max_temp)
# current_temp = sense.get_temperature()
# pixels = g.render(current_temp)
# sense.set_pixels(pixels)




from sense_hat_matrix.GraphUtil import rescale
from sense_hat_matrix.GraphUtil import temp_colour
from sense_hat_matrix.LedMatrix import LedMatrix


class Graph:
    def __init__(self, minimum, maximum):
        self.min = minimum
        self.max = maximum
        self.R = [255, 0, 0]  # Red
        self.G = [0, 255, 0]  # Green
        self.B = [0, 0, 255]  # Blue
        self.O = [0, 0, 0]  # Black
        self.matrix = LedMatrix()

    def render(self, current):
        scaled_to_index = round(rescale(self.min, self.max, current)) - 1
        column = []
        for i in range(7, -1, -1):
            if i <= scaled_to_index:
                colour = temp_colour(i, self.B, self.G, self.R)
                dot = colour
            else:
                dot = self.O
            column.append(dot)
        self.matrix.add_column_to_right(column)
        return self.matrix.render()




from sense_hat_matrix.LedMatrixUtil import MatrixType
from sense_hat_matrix.LedMatrixUtil import ListType
from sense_hat_matrix.LedMatrixUtil import list_to_matrix
from sense_hat_matrix.LedMatrixUtil import matrix_to_list
from sense_hat_matrix.LedMatrixUtil import shift_matrix_to_left
from sense_hat_matrix.LedMatrixUtil import set_column


# LED Matrix assumes an 8 x 8 LED panel which you can find on the pi's sense-hat
class LedMatrix:
    matrix: MatrixType

    def __init__(self):
        O = [0, 0, 0]  # Black
        self.matrix = list_to_matrix([
            O, O, O, O, O, O, O, O,
            O, O, O, O, O, O, O, O,
            O, O, O, O, O, O, O, O,
            O, O, O, O, O, O, O, O,
            O, O, O, O, O, O, O, O,
            O, O, O, O, O, O, O, O,
            O, O, O, O, O, O, O, O,
            O, O, O, O, O, O, O, O
        ])

    def add_column_to_right(self, column) -> None:
        self.matrix = shift_matrix_to_left(self.matrix)
        self.matrix = set_column(self.matrix, 7, column)

    def render(self) -> ListType:
        return matrix_to_list(self.matrix)





def temp_colour(current, min_colour, middle_colour, max_colour):
        colour = min_colour
        if current >= 6:
            colour = max_colour
        elif current >= 2:
            colour = middle_colour
        return colour


# https://stats.stackexchange.com/questions/281162/scale-a-number-between-a-range
def rescale(minimum, maximum, current):
    new_min = 1
    new_max = 8
    original_min = minimum
    original_max = maximum
    if current <= original_min:
        return new_min
    elif current >= original_max:
        return new_max
    else:
        return (current - original_min) * (new_max - new_min) / (original_max - original_min) + new_min




import sys
from argparse import ArgumentParser
from importlib.metadata import version


def entry_point():
    parser = ArgumentParser()
    parser.add_argument("-v", "--version", action="store_true")

    args = parser.parse_args()

    if args.version:
        print(version("sense_hat_matrix"))
        sys.exit()



import unittest

from sense_hat_matrix.Graph import Graph
from test.PrettyPrint import pretty_print_list


R = [255, 0, 0]  # Red
G = [0, 255, 0]  # Green
B = [0, 0, 255]  # Blue
O = [0, 0, 0]


class GraphSpec(unittest.TestCase):

    def test_should_render_min_value(self):
        name_of_test = "should_render_min_value"
        graph = Graph(20, 40)
        result = graph.render(20)
        pretty_print_list(name_of_test, result)
        self.assertEqual(result, [
                O, O, O, O, O, O, O, O,
                O, O, O, O, O, O, O, O,
                O, O, O, O, O, O, O, O,
                O, O, O, O, O, O, O, O,
                O, O, O, O, O, O, O, O,
                O, O, O, O, O, O, O, O,
                O, O, O, O, O, O, O, O,
                O, O, O, O, O, O, O, B
                ])

    def test_should_render_max_value(self):
        name_of_test = "should_render_max_value"
        graph = Graph(20, 40)
        result = graph.render(40)
        pretty_print_list(name_of_test, result)
        self.assertEqual(result, [
                O, O, O, O, O, O, O, R,
                O, O, O, O, O, O, O, R,
                O, O, O, O, O, O, O, G,
                O, O, O, O, O, O, O, G,
                O, O, O, O, O, O, O, G,
                O, O, O, O, O, O, O, G,
                O, O, O, O, O, O, O, B,
                O, O, O, O, O, O, O, B
                ])

    def test_should_render_scaled_to_middle(self):
        name_of_test = "should_render_scaled_to_middle"
        graph = Graph(20, 30)
        result = graph.render(25)
        pretty_print_list(name_of_test, result)
        self.assertEqual(result, [
                O, O, O, O, O, O, O, O,
                O, O, O, O, O, O, O, O,
                O, O, O, O, O, O, O, O,
                O, O, O, O, O, O, O, O,
                O, O, O, O, O, O, O, G,
                O, O, O, O, O, O, O, G,
                O, O, O, O, O, O, O, B,
                O, O, O, O, O, O, O, B
                ])

    def test_should_render_multiple_times_and_shifting_old_values_left(self):
        name_of_test = "should_render_multiple_times_and_shifting_old_values_left"
        graph = Graph(20, 30)
        graph.render(30)
        graph.render(25)
        result = graph.render(20)
        pretty_print_list(name_of_test, result)
        self.assertEqual(result, [
                O, O, O, O, O, R, O, O,
                O, O, O, O, O, R, O, O,
                O, O, O, O, O, G, O, O,
                O, O, O, O, O, G, O, O,
                O, O, O, O, O, G, G, O,
                O, O, O, O, O, G, G, O,
                O, O, O, O, O, B, B, O,
                O, O, O, O, O, B, B, B
                ])


if __name__ == "__main__":
    unittest.main()



from sense_hat_matrix.LedMatrixUtil import list_to_matrix


def pretty_print_list(name_of_test, list_of_matrix):
    matrix = list_to_matrix(list_of_matrix)
    pretty_print(name_of_test, matrix)


def pretty_print(name_of_test, matrix):
    print(name_of_test)
    for row in matrix:
        line = ""
        for c in row:
            line += str(c)
            line += ",\t\t"
        print(line)


def debug_pretty_print(expected, actual):
    if len(expected) != len(actual):
        print("Number of rows are not the same of expected [" + len(expected) + "] and actual [" + len(actual) + "]")
    if len(expected[0]) != len(actual[0]):
        print("Number of columns are not the same of expected [" + len(expected[0]) + "] and actual [" + len(actual[0]) + "]")

    for r in range(0, len(expected)):
        line = ""
        for c in range(0, len(expected[r])):
            e = expected[r][c]
            a = actual[r][c]



#!/usr/bin/env python

from setuptools import setup

# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

if __name__ == "__main__":
    setup(long_description=long_description, long_description_content_type='text/markdown')




import numpy as np

ListType = list[list[float]]
MatrixType = list[list[list[float]]]


def list_to_matrix(list_as_matrix: ListType) -> MatrixType:
    #         0   1   2   3   4   5   6   7
    matrix = [[], [], [], [], [], [], [], []]
    for r in range(0, 8):
        temp_array = []
        for c in range(0, 8):
            if r == 0:
                index = c
            else:
                index = r * 8 + c
            temp_array.append(list_as_matrix[index])
        matrix[r] = temp_array
    return matrix


def matrix_to_list(matrix: MatrixType) -> ListType:
    list_as_matrix = []
    for row in matrix:
        for e in row:
            list_as_matrix.append(e)
    return list_as_matrix


def shift_matrix_to_left(matrix: MatrixType) -> MatrixType:
    o = [0, 0, 0]  # Black
    for r in range(0, len(matrix)):
        matrix[r] = __shift_array(matrix[r], o)
    return matrix


def set_column(matrix: MatrixType, column_index: int, column: ListType):
    for r in range(0, len(matrix)):
        matrix[r][column_index] = column[r]
    return matrix


def __shift_array(array, default):
    size = len(array)
    for i in range(0, size):
        if i == size - 1:
            temp = default
        else:
            temp = array[i+1]
        array[i] = temp
    return array
